@@ -36,8 +36,9 @@
 #include <eel/eel-string.h>
 #include <eel/eel-debug.h>
 #include <libgnome/gnome-util.h>
+#include <glib.h>
 #include <glib/gi18n.h>
-#include <glib/gstrfuncs.h>
+#include <glib/gstdio.h>
 #include <gio/gio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -66,7 +67,10 @@ nautilus_compute_title_for_location (GFile *location)
 	title = NULL;
 	if (location) {
 		file = nautilus_file_get (location);
-		title = nautilus_file_get_display_name (file);
+		title = nautilus_file_get_description (file);
+		if (title == NULL) {
+			title = nautilus_file_get_display_name (file);
+		}
 		nautilus_file_unref (file);
 	}
 
@@ -95,7 +99,7 @@ nautilus_get_user_directory (void)
 					   NULL);
 	
 	if (!g_file_test (user_directory, G_FILE_TEST_EXISTS)) {
-		mkdir (user_directory, DEFAULT_NAUTILUS_DIRECTORY_MODE);
+		g_mkdir (user_directory, DEFAULT_NAUTILUS_DIRECTORY_MODE);
 		/* FIXME bugzilla.gnome.org 41286: 
 		 * How should we handle the case where this mkdir fails? 
 		 * Note that nautilus_application_startup will refuse to launch if this 
@@ -427,7 +431,7 @@ nautilus_get_desktop_directory (void)
 	/* Don't try to create a home directory */
 	if (!eel_preferences_get_boolean (NAUTILUS_PREFERENCES_DESKTOP_IS_HOME_DIR)) {
 		if (!g_file_test (desktop_directory, G_FILE_TEST_EXISTS)) {
-			mkdir (desktop_directory, DEFAULT_DESKTOP_DIRECTORY_MODE);
+			g_mkdir (desktop_directory, DEFAULT_DESKTOP_DIRECTORY_MODE);
 			/* FIXME bugzilla.gnome.org 41286: 
 			 * How should we handle the case where this mkdir fails? 
 			 * Note that nautilus_application_startup will refuse to launch if this 
@@ -520,7 +524,7 @@ nautilus_create_templates_directory (void)
 
 	dir = nautilus_get_templates_directory ();
 	if (!g_file_test (dir, G_FILE_TEST_EXISTS)) {
-		mkdir (dir, DEFAULT_NAUTILUS_DIRECTORY_MODE);
+		g_mkdir (dir, DEFAULT_NAUTILUS_DIRECTORY_MODE);
 	}
 	g_free (dir);
 }
@@ -547,7 +551,7 @@ nautilus_get_searches_directory (void)
 	g_free (user_dir);
 	
 	if (!g_file_test (searches_dir, G_FILE_TEST_EXISTS))
-		mkdir (searches_dir, DEFAULT_NAUTILUS_DIRECTORY_MODE);
+		g_mkdir (searches_dir, DEFAULT_NAUTILUS_DIRECTORY_MODE);
 
 	return searches_dir;
 }
@@ -896,6 +900,13 @@ nautilus_find_file_insensitive_next (GFile *parent, const gchar *name)
 	GFile *file;
 	const char *child_name, *compare_key;
 
+	/* First check the given version */
+	file = g_file_get_child (parent, name);
+	if (g_file_query_exists (file, NULL)) {
+		return file;
+	}
+	g_object_unref (file);
+	
 	ascii_collation_key = g_ascii_strdown (name, -1);
 	use_utf8 = g_utf8_validate (name, -1, NULL);
 	utf8_collation_key = NULL;	
@@ -905,42 +916,37 @@ nautilus_find_file_insensitive_next (GFile *parent, const gchar *name)
 		g_free (case_folded_name);
 	}
 
-	/* First check the given version */
-	file = g_file_get_child (parent, name);
-	if (g_file_query_exists (file, NULL)) {
-		return file;
-	}
-	g_object_unref (file);
-	
 	/* Enumerate and compare insensitive */
 	filename = NULL;
 	children = g_file_enumerate_children (parent,
 	                                      G_FILE_ATTRIBUTE_STANDARD_NAME,
 	                                      0, NULL, NULL);
-	while ((info = g_file_enumerator_next_file (children, NULL, NULL))) {
-		child_name = g_file_info_get_name (info);
-		
-		if (use_utf8 && g_utf8_validate (child_name, -1, NULL)) {
-			gchar *case_folded;
+	if (children != NULL) {
+		while ((info = g_file_enumerator_next_file (children, NULL, NULL))) {
+			child_name = g_file_info_get_name (info);
 			
-			case_folded = g_utf8_casefold (child_name, -1);
-			child_key = g_utf8_collate_key (case_folded, -1);
-			g_free (case_folded);
-			compare_key = utf8_collation_key;
-		} else {
-			child_key = g_ascii_strdown (child_name, -1);
-			compare_key = ascii_collation_key;
-		}
-
-		found = strcmp (child_key, compare_key) == 0;
-		g_free (child_key);
-		if (found) {
-			filename = g_strdup (child_name);
-			break;
+			if (use_utf8 && g_utf8_validate (child_name, -1, NULL)) {
+				gchar *case_folded;
+				
+				case_folded = g_utf8_casefold (child_name, -1);
+				child_key = g_utf8_collate_key (case_folded, -1);
+				g_free (case_folded);
+				compare_key = utf8_collation_key;
+			} else {
+				child_key = g_ascii_strdown (child_name, -1);
+				compare_key = ascii_collation_key;
+			}
+			
+			found = strcmp (child_key, compare_key) == 0;
+			g_free (child_key);
+			if (found) {
+				filename = g_strdup (child_name);
+				break;
+			}
 		}
+		g_file_enumerator_close (children, NULL, NULL);
+		g_object_unref (children);
 	}
-	g_file_enumerator_close (children, NULL, NULL);
-	g_object_unref (children);
 	
 	g_free (ascii_collation_key);
 	g_free (utf8_collation_key);
@@ -954,6 +960,22 @@ nautilus_find_file_insensitive_next (GFile *parent, const gchar *name)
 	return NULL;
 }
 
+gboolean
+nautilus_is_file_roller_installed (void)
+{
+	static int installed = - 1;
+
+	if (installed < 0) {
+		if (g_find_program_in_path ("file-roller")) {
+			installed = 1;
+		} else {
+			installed = 0;
+		}
+	}
+
+	return installed > 0 ? TRUE : FALSE;
+}
+
 #if !defined (NAUTILUS_OMIT_SELF_CHECK)
 
 void
