@@ -77,7 +77,9 @@ typedef enum {
 	NAUTILUS_FILE_ICON_FLAGS_IGNORE_VISITING = (1<<1),
 	NAUTILUS_FILE_ICON_FLAGS_EMBEDDING_TEXT = (1<<2),
 	NAUTILUS_FILE_ICON_FLAGS_FOR_DRAG_ACCEPT = (1<<3),
-	NAUTILUS_FILE_ICON_FLAGS_FOR_OPEN_FOLDER = (1<<4)
+	NAUTILUS_FILE_ICON_FLAGS_FOR_OPEN_FOLDER = (1<<4),
+	/* whether the thumbnail size must match the display icon size */
+	NAUTILUS_FILE_ICON_FLAGS_FORCE_THUMBNAIL_SIZE = (1<<5)
 } NautilusFileIconFlags;	
 
 /* Emblems sometimes displayed for NautilusFiles. Do not localize. */ 
@@ -87,6 +89,7 @@ typedef enum {
 #define NAUTILUS_FILE_EMBLEM_NAME_TRASH "trash"
 #define NAUTILUS_FILE_EMBLEM_NAME_NOTE "note"
 #define NAUTILUS_FILE_EMBLEM_NAME_DESKTOP "desktop"
+#define NAUTILUS_FILE_EMBLEM_NAME_SHARED "shared"
 
 typedef void (*NautilusFileCallback)          (NautilusFile  *file,
 				               gpointer       callback_data);
@@ -222,6 +225,10 @@ void                    nautilus_file_set_attributes                    (Nautilu
 									 gpointer                        callback_data);
 GFilesystemPreviewType  nautilus_file_get_filesystem_use_preview        (NautilusFile *file);
 
+char *                  nautilus_file_get_filesystem_id                 (NautilusFile                   *file);
+
+NautilusFile *          nautilus_file_get_trash_original_file           (NautilusFile                   *file);
+
 /* Permissions. */
 gboolean                nautilus_file_can_get_permissions               (NautilusFile                   *file);
 gboolean                nautilus_file_can_set_permissions               (NautilusFile                   *file);
@@ -323,6 +330,14 @@ void                    nautilus_file_set_integer_metadata              (Nautilu
 									 int                             default_metadata,
 									 int                             metadata);
 
+#define UNDEFINED_TIME ((time_t) (-1))
+
+time_t                  nautilus_file_get_time_metadata                 (NautilusFile                  *file,
+									 const char                    *key);
+void                    nautilus_file_set_time_metadata                 (NautilusFile                  *file,
+									 const char                    *key,
+									 time_t                         time);
+
 
 /* Attributes for file objects as user-displayable strings. */
 char *                  nautilus_file_get_string_attribute              (NautilusFile                   *file,
