@@ -28,8 +28,7 @@
 
 #include <glib/gi18n.h>
 
-#include <gtk/gtkmain.h>
-#include <gtk/gtkwidget.h>
+#include <gtk/gtk.h>
 
 #include <libgnome/gnome-program.h>
 #include <libgnomeui/gnome-ui-init.h>
